# This file was auto-generated by Fern from our API Definition.

import typing

ConfigurationSupportedInputFormatsItem = typing.Union[
    typing.Literal[
        "OPEN_QASM_V1",
        "OPEN_QASM_V2",
        "OPEN_QASM_V3",
        "QIR_V1",
        "BRAKET_OPEN_QASM_V3",
        "BRAKET_AHS_PROGRAM",
        "IONQ_CIRCUIT_V1",
        "QISKIT",
        "QOQO",
        "PERCEVAL",
    ],
    typing.Any,
]
