# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        CreateJobRequestInput,
        CreateJobRequestInputFormat,
        CreateJobRequestInputParams,
        CreateJobRequestInputParams_AwsIonqAria,
        CreateJobRequestInputParams_AwsIonqAria2,
        CreateJobRequestInputParams_AwsIonqForte,
        CreateJobRequestInputParams_AwsIqmGarnet,
        CreateJobRequestInputParams_AwsRigettiAnkaa,
        CreateJobRequestInputParams_AwsSimDm1,
        CreateJobRequestInputParams_AwsSimSv1,
        CreateJobRequestInputParams_AzureIonqSimulator,
        CreateJobRequestInput_AwsIonqAria,
        CreateJobRequestInput_AwsIonqAria2,
        CreateJobRequestInput_AwsIonqForte,
        CreateJobRequestInput_AwsIqmGarnet,
        CreateJobRequestInput_AwsRigettiAnkaa,
        CreateJobRequestInput_AwsSimDm1,
        CreateJobRequestInput_AwsSimSv1,
        CreateJobRequestInput_AzureIonqSimulator,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "CreateJobRequestInput": ".types",
    "CreateJobRequestInputFormat": ".types",
    "CreateJobRequestInputParams": ".types",
    "CreateJobRequestInputParams_AwsIonqAria": ".types",
    "CreateJobRequestInputParams_AwsIonqAria2": ".types",
    "CreateJobRequestInputParams_AwsIonqForte": ".types",
    "CreateJobRequestInputParams_AwsIqmGarnet": ".types",
    "CreateJobRequestInputParams_AwsRigettiAnkaa": ".types",
    "CreateJobRequestInputParams_AwsSimDm1": ".types",
    "CreateJobRequestInputParams_AwsSimSv1": ".types",
    "CreateJobRequestInputParams_AzureIonqSimulator": ".types",
    "CreateJobRequestInput_AwsIonqAria": ".types",
    "CreateJobRequestInput_AwsIonqAria2": ".types",
    "CreateJobRequestInput_AwsIonqForte": ".types",
    "CreateJobRequestInput_AwsIqmGarnet": ".types",
    "CreateJobRequestInput_AwsRigettiAnkaa": ".types",
    "CreateJobRequestInput_AwsSimDm1": ".types",
    "CreateJobRequestInput_AwsSimSv1": ".types",
    "CreateJobRequestInput_AzureIonqSimulator": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateJobRequestInput",
    "CreateJobRequestInputFormat",
    "CreateJobRequestInputParams",
    "CreateJobRequestInputParams_AwsIonqAria",
    "CreateJobRequestInputParams_AwsIonqAria2",
    "CreateJobRequestInputParams_AwsIonqForte",
    "CreateJobRequestInputParams_AwsIqmGarnet",
    "CreateJobRequestInputParams_AwsRigettiAnkaa",
    "CreateJobRequestInputParams_AwsSimDm1",
    "CreateJobRequestInputParams_AwsSimSv1",
    "CreateJobRequestInputParams_AzureIonqSimulator",
    "CreateJobRequestInput_AwsIonqAria",
    "CreateJobRequestInput_AwsIonqAria2",
    "CreateJobRequestInput_AwsIonqForte",
    "CreateJobRequestInput_AwsIqmGarnet",
    "CreateJobRequestInput_AwsRigettiAnkaa",
    "CreateJobRequestInput_AwsSimDm1",
    "CreateJobRequestInput_AwsSimSv1",
    "CreateJobRequestInput_AzureIonqSimulator",
]
