# This file was auto-generated by Fern from our API Definition.

import typing

BackendHardwareProvider = typing.Union[
    typing.Literal[
        "IONQ",
        "RIGETTI",
        "OQC",
        "AWS",
        "AZURE",
        "IBM",
        "DWAVE",
        "QRYD",
        "QUERA",
        "IQM",
        "QUDORA",
        "QUANTINUUM",
        "QUANDELA",
    ],
    typing.Any,
]
