# This file was auto-generated by Fern from our API Definition.

import typing

BackendTechnology = typing.Union[
    typing.Literal["SUPERCONDUCTING", "TRAPPED_ION", "PHOTONIC", "NEUTRAL_ATOMS", "UNKNOWN"], typing.Any
]
