# This file was auto-generated by Fern from our API Definition.

import typing

BackendProvider = typing.Union[
    typing.Literal["AZURE", "AWS", "IBM", "IBM_CLOUD", "DWAVE", "TSYSTEMS", "QRYD", "QUDORA", "QUANDELA", "IQM"],
    typing.Any,
]
