# This file was auto-generated by Fern from our API Definition.

import typing

CostGranularity = typing.Union[
    typing.Literal["shot", "job", "1-Qubit Gate Shot", "2-Qubit Gate Shot", "minute", "second", "millisecond"],
    typing.Any,
]
