# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .json_node import JsonNode


class CalibrationResponse(UniversalBaseModel):
    backend_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="backendId")] = pydantic.Field(
        default=None
    )
    """
    Unique identifier of the backend
    """

    calibrated_at: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="calibratedAt")] = None
    calibration: typing.Optional[JsonNode] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
