# This file was auto-generated by Fern from our API Definition.

import typing

DocumentationSdksItem = typing.Union[
    typing.Literal["PLANQK", "PENNYLANE", "QISKIT", "QISKIT_RUNTIME", "DWAVE", "QOQO", "PERCEVAL"], typing.Any
]
