# This file was auto-generated by Fern from our API Definition.

import typing

AzureIonqJobInputCircuitItemType = typing.Union[
    typing.Literal[
        "x",
        "y",
        "z",
        "rx",
        "ry",
        "rz",
        "h",
        "s",
        "si",
        "v",
        "vi",
        "t",
        "ti",
        "not",
        "cnot",
        "swap",
        "xx",
        "yy",
        "zz",
        "pauliExp",
    ],
    typing.Any,
]
