# This file was auto-generated by Fern from our API Definition.

import typing

JobStatus = typing.Union[
    typing.Literal["UNKNOWN", "PENDING", "ABORTED", "RUNNING", "COMPLETED", "FAILED", "CANCELLING", "CANCELLED"],
    typing.Any,
]
